# Copyright (C) 2007-2008 Jean-Baptiste LAMY -- jibalamy@free.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later verson.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, os.path, getpass, shutil, time

import cackompile
from cackompile.globdef import *


class CompileError(StandardError): pass

next_step = 1

def do_python(step_name):
  global next_step
  step = next_step
  next_step += 1
  if (cackompile.globdef.start_at_step > step) or (step > cackompile.globdef.up_to_step):
    print "* Cackompile * step %s: %s (skipped)" % (step, step_name)
    return 0
  
  else:
    print "* Cackompile * step %s: %s..." % (step, step_name)
    return step
  
def do_sh(command, step_name = ""):
  step = do_python(step_name or "run '%s' in '%s'" % (command, os.getcwd()))
  if step:
    log = os.path.join(log_dir, "cackompile_log_step_%s" % step)
    r = os.system(command + " > " + log + " 2>&1")
    if r != 0:
      print "* Cackompile * ERROR at step %s! look at %s" % (step, log)
      sys.exit(1)
    return step
  
def cd(directory): os.chdir(directory)

def cp(src, dest): shutil.copy(src, dest)

def cp_tree(src, dest):
  src  = os.path.abspath(src )
  dest = os.path.abspath(dest)
  
  for dir, sub_dirs, files in os.walk(src):
    for file in files:
      src_file  = os.path.join(dir, file)
      dest_file = src_file.replace(src, dest)
      mkdirhier(os.path.dirname(dest_file))
      if os.path.islink(src_file):
        ln(os.readlink(src_file), dest_file)
      else:
        cp(src_file, dest_file)
      
def mv(src, dest): os.rename(src, dest)

def ln(src, dest):
  try: rm(dest)
  except: pass
  os.symlink(src, dest)

def rm(*files):
  for file in files:
    if   os.path.islink(file):
      os.unlink(file)
    elif  os.path.exists(file):
      if os.path.isdir(file): shutil.rmtree(file)
      else:                   os.unlink(file)
      
def mkdirhier(dir):
  if not os.path.exists(dir): os.makedirs(dir)

class Environment(dict):
  def __init__(self, **kargs):
    dict.__init__(self, os.environ)
    self.update(kargs)
    
  def make_current(self):
    for k, v in self.items():
      os.environ[k] = v
    for k, v in os.environ.items():
      if not self.has_key(k): del os.environ[k]
    

host_env      = Environment()
target_env295 = Environment()
target_env295["QPEDIR"         ] = "/opt/Qtopia/sharp"
target_env295["QTDIR"          ] = "/opt/Qtopia/sharp"
target_env295["PATH"           ] = "/opt/QtPalmtop/bin/config:/opt/Embedix/tools/arm-linux/bin:/opt/Qtopia/sharp/bin:/opt/Embedix/tools/bin:" + target_env295["PATH"]
target_env295["TMAKEPATH"      ] = "/opt/Qtopia/tmake/lib/qws/linux-sharp-g++/"
target_env295["CC"             ] = "arm-linux-gcc"
target_env295["CXX"            ] = "arm-linux-g++"
target_env295["CPPFLAGS"       ] = "-I" + os.path.join(target_prefix, "include") + " -L" + os.path.join(target_prefix, "lib")
target_env295["CFLAGS"         ] = target_env295["CPPFLAGS" ]
target_env295["PKG_CONFIG_PATH"] = "/opt/QtPalmtop/lib/pkgconfig/"
target_env295["PKG_CONFIG"     ] = "/usr/bin/pkg-config"

# + /opt/Embedix/tools/arm-linux/include/
# + /opt/Embedix/tools/arm-linux/include/g++-3/
# -I/home/jiba/src/cackompile/root/opt/QtPalmtop/include/ -I/tmp/qt-2.3.2/include -L/home/jiba/src/cackompile/root/opt/QtPalmtop/lib -L/tmp/qt-2.3.2/lib

target_env3   = Environment()
target_env3  ["PATH"           ] = "/opt/QtPalmtop/bin/config:/opt/cross/arm/3.4.5-xscale-fpa/bin/:" + target_env3["PATH"]
target_env3  ["CC"             ] = "armv5tel-cacko-linux-gcc"
target_env3  ["CXX"            ] = "armv5tel-cacko-linux-g++"
target_env3  ["CPPFLAGS"       ] = "-I" + os.path.join(target_prefix, "include") + " -L" + os.path.join(target_prefix, "lib") + " -I/opt/cross/arm/3.4.5-xscale-fpa/armv5tel-cacko-linux/include/ -L/opt/cross/arm/3.4.5-xscale-fpa/armv5tel-cacko-linux/lib/"
target_env3  ["CFLAGS"         ] = target_env3["CPPFLAGS" ]
target_env3  ["PKG_CONFIG_PATH"] = "/opt/QtPalmtop/lib/pkgconfig/"
target_env3  ["PKG_CONFIG"     ] = "/usr/bin/pkg-config"

# PATH : /opt/cross/arm/3.4.5-xscale-fpa/hack_bin/

target_envs         = {"2.95" : target_env295, "3" : target_env3 }
target_bin_prefixes = {"2.95" : "arm-linux-", "3" : "armv5tel-cacko-linux-" }

def get_include_dir():
  dirs = []
  for dir in target_envs[self.cross_gcc_version]["CFLAGS"].split():
    if dir.startswith("-I"): dirs.append(dir[2:])
  return dirs

def get_lib_dir():
  dirs = []
  for dir in target_envs[self.cross_gcc_version]["CFLAGS"].split():
    if dir.startswith("-L"): dirs.append(dir[2:])
  return dirs

class Builder(object):
  def __init__(self, tarball, name, version, archive_type):
    self.tarball           = tarball
    self.name              = name
    self.package_name      = name
    self.version           = version
    self.archive_type      = archive_type
    self.versioned_name    = "%s-%s" % (self.name, self.version)
    self.build_version     = "cackompile" + cackompile_version
    self.control_values    = {}
    self.cross_gcc_version = "3" # "2.95" or "3"
    self.dependencies      = []
    self.install_in_root   = 1
    self.nand_flash_only   = 0
    self.time0             = int(time.time())
    time.sleep(2) # Ensure that current time is superior to time0
    
  def build(self):
    self.clean_previous_install()
    self.build_for_target()
    self.index_and_clean()
    self.create_ipkg()
    
  def clean_previous_install(self):
    index_file = os.path.join(indexes_dir, self.versioned_name)
    if os.path.exists(index_file) and do_python("clean previous install"):
      for old_file in open(index_file).read().split("\n"):
        if old_file.startswith(target_prefix): rm(old_file)
        
  def build_for_host(self):
    host_env.make_current()
    cd(host_tmp_dir)
    self.extract_for_host()
    
  def build_for_target(self):
    target_envs[self.cross_gcc_version].make_current()
    cd(target_tmp_dir)
    self.extract_for_target()
    
  def extract_for_host(self): self.extract()
  
  def extract_for_target(self): self.extract()
  
  def extract(self):
    if   self.archive_type == "tar.gz" : do_sh("tar -xvzf %s" % self.tarball)
    elif self.archive_type == "tar.bz2": do_sh("tar -xvjf %s" % self.tarball)
    elif self.archive_type == "zip"    : do_sh("unzip %s"     % self.tarball)

  def apply_patches(self): pass
  
  def apply_patch(self, patch, nb_strips = 2):
    do_sh("patch --strip %s < %s" % (nb_strips, patch))
    
  def index_and_clean(self):
    self.rename_binaries()
    self.index_files()
    self.remove_static_lib()
    self.strip_binaries()
    self.link_config_bin()
    
  def rename_binaries(self):
    if do_python("rename binaries arm-linux-XXX or armv5tel-cacko-linux-XXX in XXX"):
      bin_dir = os.path.join(target_prefix, "bin")
      if os.path.exists(bin_dir):
        for binary in os.listdir(bin_dir):
          file = os.path.join(bin_dir, binary)
          if   binary.startswith("armv5tel-cacko-linux-"):
            mv(file, os.path.join(bin_dir, binary[len("armv5tel-cacko-linux-"):]))
          elif binary.startswith("arm-linux-"):
            mv(file, os.path.join(bin_dir, binary[len("arm-linux-"):]))
            
        for binary in os.listdir(bin_dir):
          file = os.path.join(bin_dir, binary)
          if   os.path.islink(file):
            link_src = os.readlink(file)
            link_src2 = link_src.replace("armv5tel-cacko-linux-", "").replace("arm-linux-", "")
            if link_src2 != link_src:
              rm(file)
              ln(link_src2, file)
              
  def is_devel_file(self, file):
    if ("/include/" in file) or ("/pkgconfig/" in file) or ("/test/" in file) or ("/aclocal/" in file): return 1
    if ("/bin/" in file) and (file.endswith("-config")): return 1
    if ("/lib/" in file) and (file.endswith(".a") or file.endswith(".la")): return 1
    if file.endswith(".py" ) and os.path.exists(file      + "o"): return 1
    if file.endswith(".pyc") and os.path.exists(file[:-1] + "o"): return 1
    
  def is_doc_file(self, file):
    if ("/demo/" in file) or ("/doc/" in file) or ("/gtk-doc/" in file) or ("/gnome/html/" in file) or ("/info/" in file) or ("/man/" in file) or ("/examples/" in file): return 1
    
  def index_files(self):
    #print os.path.getmtime("/opt/QtPalmtop/lib/locale")
    
    self.all_files = []
    for dir, sub_dirs, files in os.walk(target_prefix):
      for sub_dir in sub_dirs:
        sub_dir = os.path.join(dir, sub_dir)
        if os.path.islink(sub_dir) and (os.path.getmtime(sub_dir) > self.time0):
          self.all_files.append(sub_dir)
          
      for file in files:
        file = os.path.join(dir, file)
        if os.lstat(file).st_mtime > self.time0:
          self.all_files.append(file)
          
    if do_python("index new files (%s found)" % len(self.all_files)):
      open(os.path.join(indexes_dir, self.versioned_name          ), "w").write("\n".join(self.all_files))
      
  def index_ipkg_files(self):
    self.ipkg_files = [file for file in self.all_files if (not self.is_devel_file(file)) and (not self.is_doc_file(file))]
    
    if do_python("index ipkg files (%s found)" % len(self.ipkg_files)):
      # Order ipkg files, so as symlinks are AFTER the file they link to (this is required by the tar program on the Zaurus)
      def link_depth(file):
        if os.path.islink(file): return 1 + link_depth(os.path.join(os.path.dirname(file), os.readlink(file)))
        else: return 0
      self.ipkg_files.sort(lambda a, b: cmp(link_depth(a), link_depth(b)))
      
      if not self.nand_flash_only:
        # File names in the ipkg package must begin with "." (e.g. "./opt/XXX" instead of "/opt/XXX"), else the package cannot be installed on cards.
        self.ipkg_files = ["." + file for file in self.ipkg_files]
        
      # We need to add directories in the ipkg too
      self.ipkg_dirs  = set()
      def add_dirs(file):
        file = os.path.dirname(file)
        if file and (file != ".") and (file != "/"):
          self.ipkg_dirs.add(file)
          add_dirs(file)
      for file in self.ipkg_files: add_dirs(file)
      self.ipkg_dirs = list(self.ipkg_dirs)
      self.ipkg_dirs.sort()
      
      open(os.path.join(indexes_dir, self.versioned_name + "_ipkg"), "w").write("\n".join(self.ipkg_dirs) + "\n" + "\n".join(self.ipkg_files))
      
  def remove_static_lib(self):
    static_libs = []
    for file in self.all_files:
      if file.endswith(".a") and self.is_devel_file(file): static_libs.append(file)
      
    if do_python("remove static libs (%s found)" % len(static_libs)):
      for file in static_libs:
        self.all_files.remove(file)
        rm(file)
        
  def strip_binaries(self):
    binary_files = []
    for file in self.all_files:
      if   file.endswith(".so"): binary_files.append(file)
      elif file.endswith(".o" ): continue
      elif file.endswith(".sh"): continue
      elif file.endswith(".py"): continue
      elif os.access(file, os.X_OK) and " ELF 32-bit " in os.popen("file %s" % file).read(): binary_files.append(file)
    if binary_files: do_sh(target_bin_prefixes[self.cross_gcc_version] + "strip %s" % " ".join(binary_files), step_name = "strip binaries (%s found)" % len(binary_files))
    
  def link_config_bin(self):
    config_bins = []
    for file in self.all_files:
      if ("/bin/" in file) and (not "/bin/config/" in file) and file.endswith("-config"): config_bins.append(file)
      
    if do_python("link config binaries (%s found)" % len(config_bins)):
      for file in config_bins:
        link_to = os.path.join(target_prefix, "bin", "config", os.path.basename(file))
        rm(link_to)
        ln(file, link_to)
        
  def create_ipkg(self):
    self.index_ipkg_files()
    self.create_empty_ipkg()
    self.create_control_file()
    self.create_scripts()
    self.create_ipkg_data()
    self.assemble_ipkg()
    print "* Cackompile * %s/%s is ready! Be careful it's still hot!" % (cackompile.globdef.ipkg_dir, self.ipkg_name)
    
  def create_empty_ipkg(self):
    self.control_dir  = os.path.join(target_tmp_dir, self.versioned_name + "_CONTROL")
    self.ipkg_dir     = os.path.join(target_tmp_dir, self.versioned_name + "_IPKG")
    self.control_file = os.path.join(self.control_dir, "control")
    self.ipkg_name    = "%s_%s-%s_arm.ipk"% (self.package_name, self.version, self.build_version)
    
    if do_python("create empty ipkg (in %s and %s)" % (self.control_dir, self.ipkg_dir)):
      for required_directory in [self.control_dir, self.ipkg_dir]:
        if not os.path.exists(required_directory):
          os.makedirs(required_directory)
      open(os.path.join(self.ipkg_dir, "debian-binary"), "w").write("2.0\n")
      
  def create_control_file(self):
    if do_python("create control file (%s)" % (self.control_file)):
      values = {
        "Package"      : self.package_name,
        "Filename"     : self.ipkg_name,
        "Version"      : "%s-%s" % (self.version, self.build_version),
        "Maintainer"   : getpass.getuser().title(),
        "Architecture" : "arm",
        "Priority"     : "optional",
        "Section"      : "unknown",
        "Description"  : "%s version %s compiled with Cackompile." % (self.name, self.version)
        }
      if self.dependencies:
        values["Depends"] = ",".join(self.dependencies)
      values.update(self.control_values)
      open(self.control_file, "w").write("\n".join([k + ": " + v for k, v in values.items()]))
      
  def create_scripts(self): pass # preinst, postinst, prerm, and postrm
  
  def create_ipkg_data(self):
    cd("/")
    #exclude = " ".join(["--exclude %s" % i for i in self.excludes])
    #do_sh("tar %s -cvzf %s/data.tar.gz ./opt/QtPalmtop" % (exclude, self.ipkg_dir))
    do_sh("tar --no-recursion -cvzf %s/data.tar.gz --files-from=%s" % (self.ipkg_dir, os.path.join(indexes_dir, self.versioned_name + "_ipkg")))
    
  def assemble_ipkg(self):
    cd(self.control_dir)
    do_sh("tar -cvzf %s/control.tar.gz ./*" % self.ipkg_dir)
    cd(self.ipkg_dir)
    do_sh("tar -cvzf %s/%s ./*" % (cackompile.globdef.ipkg_dir, self.ipkg_name))
    

class LocalDataBuilder(Builder):
  def __init__(self, name, version, local_dir):
    Builder.__init__(self, "", name, version, "")
    
    self.local_dir = local_dir
    
  def build_for_target(self):
    cp_tree(self.local_dir, target_prefix)
    
  
class ConfigureMakeMakeInstallBuilder(Builder):
  def build_for_host(self):
    Builder.build_for_host(self)
    
    cd(os.path.join(host_tmp_dir, self.versioned_name))
    self.configure_for_host()
    self.apply_patches()
    self.make_for_host()
    
  def build_for_target(self):
    Builder.build_for_target(self)
    
    cd(os.path.join(target_tmp_dir, self.versioned_name))
    self.configure_for_target()
    self.apply_patches()
    self.make_for_target()
    self.make_install_for_target()
    
  def configure_for_host(self, extra = ""):
    do_sh("./configure %s" % extra)
    
  def make_for_host(self):
    do_sh("make")
    
  def configure_for_target(self, extra = ""):
    if self.cross_gcc_version == "3":
      do_sh("./configure --prefix=%s --program-transform-name=s,y,y, --build=i686-linux --host=armv5tel-cacko-linux --target=armv5tel-cacko-linux %s" % (target_prefix, extra)) # from http://www.oesf.org/index.php?title=Cross_compiling_%28non-Qtopia%29_tips
    else:
      do_sh("./configure --prefix=%s --program-transform-name=s,y,y, --build=i686-linux --host=arm-linux --target=arm-linux %s" % (target_prefix, extra)) # from http://www.oesf.org/index.php?title=Cross_compiling_%28non-Qtopia%29_tips
      
  def hack_libtool(self):
    if os.path.exists("./libtool"):
      if do_python("hack libtool"):
        s = open("./libtool").read()
        s = re.sub('sys_lib_search_path_spec=".*?"'  , 'sys_lib_search_path_spec="%s"'   % get_include_dir, s)
        s = re.sub('sys_lib_dlsearch_path_spec=".*?"', 'sys_lib_dlsearch_path_spec="/lib /usr/lib /usr/lib.rom /opt/QtPalmtop/lib"' % s, s)
        open("./libtool", "w").write(s)
        
  def make_for_target(self):
    do_sh("make")
    
  def make_install_for_target(self):
    do_sh("make install")
    

