# Copyright (C) 2007-2008 Jean-Baptiste LAMY -- jibalamy@free.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later verson.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, os.path, shutil, stat

from cackompile         import *
from cackompile.globdef import *
from cackompile.base    import *

class SpecificBuilder(ConfigureMakeMakeInstallBuilder):
  def __init__(self, tarball, name, version, archive_type):
    ConfigureMakeMakeInstallBuilder.__init__(self, tarball, name, version, archive_type)
    
    self.major_version  = self.version.rsplit(".", 1)[0]
    self.build_version += ".1"
    self.control_values["Section"] = "libs"
    self.control_values["Description"] = "Python is a dynamic programming langage. This package includes the Python interpreter with all the standard modules, except Tkinter and Ctypes."
    self.cross_gcc_version = "3"
    
  def build(self):
    self.build_for_host()
    ConfigureMakeMakeInstallBuilder.build(self)
    
  def configure_for_target(self):
    if do_python("hack configure"):
      s = open("./configure").read()
      s = s.replace("""
echo "$as_me:$LINENO: checking for %zd printf() format support" >&5
echo $ECHO_N "checking for %zd printf() format support... $ECHO_C" >&6
if test "$cross_compiling" = yes; then
""", """
echo "$as_me:$LINENO: checking for %zd printf() format support" >&5
echo $ECHO_N "checking for %zd printf() format support... $ECHO_C" >&6
if test "$cross_compiling" = disable_this_test; then
""")
      open("./configure", "w").write(s)
      
    ConfigureMakeMakeInstallBuilder.configure_for_target(self, "--enable-shared")
    
    if do_python("hack Makefile"):
      s = open("./Makefile").read()
      
      # Use PGEN from the host compilation
      s = s.replace("""PGEN=		Parser/pgen$(EXE)""",
                    """PGEN=		%s/Parser/pgen$(EXE)""" % os.path.join(host_tmp_dir, self.versioned_name))
      
      # Disable PGEN compilation
      s = s.replace("""PGENOBJS=	$(PGENMAIN) $(POBJS) $(PGOBJS)""",
                    """""")
      
      # Use Python from the host compilation
      s = s.replace("""./$(BUILDPYTHON)""",
                    os.path.join(host_tmp_dir, self.versioned_name, "python"))
      
      open("./Makefile", "w").write(s)


    if do_python("hack setup.py"):
      s = open("./setup.py").read()
      
      # After a module compilation, disable the importation test
      s = s.replace("""
        try:
            imp.load_dynamic(ext.name, ext_filename)
""", """
        return # Disabled for cross-compilation
        try:
            imp.load_dynamic(ext.name, ext_filename)
""")
      
      # Use the right include and lib dirs
      s = s.replace("""def detect_modules(self):""", """def detect_modules(self):
        add_dir_to_list(self.compiler.include_dirs, '%s')
        add_dir_to_list(self.compiler.library_dirs, '%s')
""" % (os.path.join(target_prefix, "include"),
       os.path.join(target_prefix, "lib")))
      
      # give the right arguments to ctype's configure
      s = s.replace("""config_args = []""",
                    """config_args = ["--build=i686-linux", "--host=arm-linux"]""")
      
      open("./setup.py", "w").write(s)
      
    if do_python("hack compile.c (force python -O)"):
      s = open("./Python/compile.c").read()
      s = s.replace("""int Py_OptimizeFlag = 0;""", """int Py_OptimizeFlag = 1;""")
      open("./Python/compile.c", "w").write(s)
      
  def is_devel_file(self, file):
    if ("/config/" in file) or ("/python-config/" in file) or ("/python2.5-config/" in file): return 1
    if ("/idlelib/" in file) or ("/lib-tk/" in file): return 1
    if ("wininst-" in file): return 1
    if file.endswith("bin/python%s" % self.major_version): return 1
    if file.endswith("bin/idle"): return 1
    return ConfigureMakeMakeInstallBuilder.is_devel_file(self, file)
      
