# -*- coding: utf-8 -*-

# Copyright (C) 2007-2008 Jean-Baptiste LAMY -- jibalamy@free.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later verson.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, os.path, shutil, stat

from cackompile         import *
from cackompile.globdef import *
from cackompile.base    import *

class SpecificBuilder(LocalDataBuilder):
  def __init__(self):
    LocalDataBuilder.__init__(self, "fr-translation-desktop", "0.2", os.path.join(os.path.dirname(__file__), "prefix"))
    
    self.build_version += ".2"
    self.control_values["Section"] = "libs"
    self.control_values["Description"] = "French translation for the desktop icons. You need to switch your Zaurus to French locale too (Language = fr in /home/zaurus/Settings/locale.conf), and to restart Qtopia or reboot after installing this package. Install in main memory only."
    self.nand_flash_only = 1
    
    self.translations = {
      "/opt/QtPalmtop/apps/Applications/.directory" : "Applications",
      "/opt/QtPalmtop/apps/Applications/advancedfm.desktop" : "Gestionnaire de fichiers",
      "/opt/QtPalmtop/apps/Applications/calculator.desktop" : "Calculatrice",
      "/opt/QtPalmtop/apps/Applications/citytime.desktop" : "Horloge internationale",
      "/opt/QtPalmtop/apps/Applications/clock.desktop" : "Horloge",
      "/opt/QtPalmtop/apps/Applications/embeddedkonsole.desktop" : "Terminal",
      "/opt/QtPalmtop/apps/Applications/filebrowser.desktop" : "Gestionnaire de fichiers",
      "/opt/QtPalmtop/apps/Applications/filesearch.desktop" : "Rechercher des fichiers",
      "/opt/QtPalmtop/apps/Applications/qpesearch.desktop" : "Rechercher des fichiers",
      "/opt/QtPalmtop/apps/Applications/hancomsheet.desktop" : "Tableur Hancom",
      "/opt/QtPalmtop/apps/Applications/hancomword.desktop" : "Traitement de texte Hancom",
      "/opt/QtPalmtop/apps/Applications/qpdf.desktop" : "Lecteur PDF",
      "/opt/QtPalmtop/apps/Applications/qflashplayer.desktop" : "Lecteur Flash",
      "/opt/QtPalmtop/apps/Applications/zeditor.desktop" : "Éditeur de texte",
      "/opt/QtPalmtop/apps/Applications/zuc.desktop" : "Unités",
      "/opt/QtPalmtop/apps/Applications/zbedic.desktop" : "Dictiionnaires",
      
      "/opt/QtPalmtop/apps/Multimedia/.directory" : "Multimedia",
      "/opt/QtPalmtop/apps/Multimedia/movieplayer.desktop" : "Lecteur vidéo",
      "/opt/QtPalmtop/apps/Multimedia/musicplayer.desktop" : "Lecteur audio",
      "/opt/QtPalmtop/apps/Multimedia/photostorage.desktop" : "Lecteur d'images",
      "/opt/QtPalmtop/apps/Multimedia/presenter.desktop" : "Présentation",
      "/opt/QtPalmtop/apps/Multimedia/voicerec.desktop" : "Enregistreur audio",
      "/opt/QtPalmtop/apps/Multimedia/zimager.desktop" : "Lecteur d'images",
      
      "/opt/QtPalmtop/apps/Network/.directory" : "Réseau",
      "/opt/QtPalmtop/apps/Network/Ob_qtmail.desktop" : "Courriel",
      "/opt/QtPalmtop/apps/Network/netfront3.desktop" : "Navigateur oueb",
      "/opt/QtPalmtop/apps/Network/opera.desktop" : "Navigateur oueb",
      "/opt/QtPalmtop/apps/Network/wlanmon.desktop" : "Réseau sans fil",
      
      "/opt/QtPalmtop/apps/Pim/.directory" : "Info perso",
      "/opt/QtPalmtop/apps/Pim/addressbook.desktop" : "Carnet d'adresses",
      "/opt/QtPalmtop/apps/Pim/datebook.desktop" : "Agenda",
      "/opt/QtPalmtop/apps/Pim/textedit.desktop" : "Notes",
      "/opt/QtPalmtop/apps/Pim/todo.desktop" : "Choses à faire",
      
      "/opt/QtPalmtop/apps/Settings/.directory" : "Réglages",
      "/opt/QtPalmtop/apps/Settings/Appearance.desktop" : "Apparence",
      "/opt/QtPalmtop/apps/Settings/Calibrate.desktop" : "Calibrer l'écran",
      "/opt/QtPalmtop/apps/Settings/CyrKeyboard.desktop" : "Clavier virtuel",
      "/opt/QtPalmtop/apps/Settings/LUSScreenSaverUtil.desktop" : "Économiseur d'écran",
      "/opt/QtPalmtop/apps/Settings/Light.desktop" : "Éclairage et alimentation",
      "/opt/QtPalmtop/apps/Settings/Security.desktop" : "Sécurité",
      "/opt/QtPalmtop/apps/Settings/appskey.desktop" : "Touches d'application",
      "/opt/QtPalmtop/apps/Settings/backup.desktop" : "Sauvegardes",
      "/opt/QtPalmtop/apps/Settings/datetime.desktop" : "Date et heure",
      "/opt/QtPalmtop/apps/Settings/irrecv.desktop" : "Réception infra-rouge",
      "/opt/QtPalmtop/apps/Settings/keyhelperconf.desktop" : "Réglage Keyhelper",
      "/opt/QtPalmtop/apps/Settings/mirror.desktop" : "Sortie VGA",
      "/opt/QtPalmtop/apps/Settings/netsetup.desktop" : "Réseau",
      "/opt/QtPalmtop/apps/Settings/qinstall.desktop" : "(Dés) installer des logiciels",
      "/opt/QtPalmtop/apps/Settings/qsamba.desktop" : "Connection USB au PC",
      "/opt/QtPalmtop/apps/Settings/quit.desktop" : "Redémarrer",
      "/opt/QtPalmtop/apps/Settings/ssoundconf.desktop" : "Son",
      "/opt/QtPalmtop/apps/Settings/sysinfo.desktop" : "Informations systèmes",
      "/opt/QtPalmtop/apps/Settings/tabconf.desktop" : "Onglets",
      "/opt/QtPalmtop/apps/Settings/zaureceive.desktop" : "Réception de données",
      "/opt/QtPalmtop/apps/Settings/zmigration.desktop" : "Migration",
      "/opt/QtPalmtop/apps/Settings/qtsamba.desktop" : "Connection au PC",
      
      "/opt/QtPalmtop/apps/XQt/.directory" : "X/Qt",
      "/opt/QtPalmtop/apps/XQt/Xqt.desktop" : "Lancer X",
      "/opt/QtPalmtop/apps/XQt/Xqtrl1.desktop" : "Serveur X/Qt",
      "/opt/QtPalmtop/apps/XQt/Xqtrl2.desktop" : "X/Qt enchâssé",
      
      "/opt/QtPalmtop/apps/__5x010file.desktop" : "Fichiers",
      "/opt/QtPalmtop/apps/__6x010netconnect.desktop" : "Connecter",
      "/opt/QtPalmtop/apps/__6x020netdisconnect.desktop" : "Déconnecter",
      "/opt/QtPalmtop/apps/__7x009rotate.desktop" : "Rotation",
      "/opt/QtPalmtop/apps/__7x010zoomin.desktop" : "Zoom +",
      "/opt/QtPalmtop/apps/__7x011zoomout.desktop" : "Zoom -",
      "/opt/QtPalmtop/apps/__8reboot.desktop" : "Redémarrer",
      "/opt/QtPalmtop/apps/__9quit.desktop" : "Suspendre",
      }
    
  def build_for_target(self): pass
    
  def strip_binaries(self): pass
  
  def create_scripts(self):
    if do_python("create scripts"):
      preinst  = "#!/bin/bash\n"
      preinst += "# Generated by Cackompile -- do not modify"
      for file, translation in self.translations.items():
        preinst += "\n"
        preinst += r"""if test -a %s ; then echo "Name[fr]=%s" >> %s ; fi""" % (file, translation, file)
      open(os.path.join(self.control_dir, "preinst"), "w").write(preinst)
      
      postrm  = "#!/bin/bash\n"
      postrm += "# Generated by Cackompile -- do not modify"
      for file, translation in self.translations.items():
        postrm += "\n"
        postrm += r"""if test -a %s ; then sed -i '/Name\[fr\]/d' %s ; fi""" % (file, file)
      open(os.path.join(self.control_dir, "postrm"), "w").write(postrm)
