# Copyright (C) 2007-2008 Jean-Baptiste LAMY -- jibalamy@free.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later verson.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, os.path, re

from cackompile         import *
from cackompile.globdef import *
from cackompile.base    import *

class SpecificBuilder(ConfigureMakeMakeInstallBuilder):
  def __init__(self, tarball, name, version, archive_type):
    ConfigureMakeMakeInstallBuilder.__init__(self, tarball, name, version, archive_type)
    
    self.build_version += ".2"
    self.control_values["Section"] = "X11"
    self.control_values["Description"] = "LyX is a text processor; it can export to LaTeX. This is LyX version 1.4.5.1, which can open files created with LyX 1.5. It has been patched for adapting the user interface to small screens (specific .ui file), and for improving the performances. With these patches, typing text with LyX on a Zaurus is very doable!"
    self.cross_gcc_version = "3"
    self.dependencies.append("libxforms")
    self.dependencies.append("python")
    
  def build_for_target(self):
    ConfigureMakeMakeInstallBuilder.build_for_target(self)
    
    self.change_ui()
    
  def configure_for_target(self):
    ConfigureMakeMakeInstallBuilder.configure_for_target(
      self,
      "--without-aiksaurus --with-frontend=xforms --x-includes=/opt/cross/arm/3.4.5-xscale-fpa/armv5tel-cacko-linux/include/ --x-libraries=/opt/cross/arm/3.4.5-xscale-fpa/armv5tel-cacko-linux/lib/ --with-extra-inc=%s --with-extra-lib=%s" % (
      os.path.join(target_prefix, "include"),
      os.path.join(target_prefix, "lib"),
      ))
    
  def apply_patches(self):
    self.apply_patch(os.path.join(os.path.dirname(__file__), "lyx-xforms-optimize-toolbars-update.patch"), 2)
    
  def change_ui(self):
    if do_python("change default User Interface file (.ui)"):
      ui_dir = os.path.join(target_prefix, "share", "lyx", "ui")
      os.rename(os.path.join(ui_dir,                    "default.ui"), os.path.join(ui_dir, "default_non_zaurus.ui"))
      cp       (os.path.join(os.path.dirname(__file__), "default.ui"), os.path.join(ui_dir, "default.ui"))
      
  def is_doc_file(self, file):
    if file.endswith("LaTeXConfig.lyx.in"): return 0
    if "splash" in file: return 0
    return ConfigureMakeMakeInstallBuilder.is_doc_file(self, file)
  
  def is_devel_file(self, file):
    if file.endswith(".py"): return 0 # Keep .py file since LyX expect them
    return ConfigureMakeMakeInstallBuilder.is_devel_file(self, file)
