# Copyright (C) 2007-2008 Jean-Baptiste LAMY -- jibalamy@free.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later verson.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys, os, os.path, shutil, stat

from cackompile         import *
from cackompile.globdef import *
from cackompile.base    import *

class SpecificBuilder(LocalDataBuilder):
  def __init__(self):
    LocalDataBuilder.__init__(self, "xqt", "0.8.2beta1", os.path.join(os.path.dirname(__file__), "prefix"))
    
    self.build_version += ".0"
    self.control_values["Section"] = "libs"
    self.control_values["Description"] = "X/Qt X server. This is a repackaging of Hd Luc / Menaie's X/Qt super Jumbo, but including only the X server part (Gtk, Gnome, Scim, Anthy, Uim,... are not included in this package)."
    self.nand_flash_only = 0
    self.dependencies.append("libiconv")
    self.dependencies.append("libexpat0")
    self.dependencies.append("libstdcxx")
    
  def build_for_target(self):
    if do_python("copy data"):
      cp_tree(self.local_dir, target_prefix)
      
  def strip_binaries(self): pass
  
  def create_scripts(self):
    if do_python("create scripts"):
      postinst = """#!/bin/bash

MODEL=`grep ^Hardware /proc/cpuinfo | head -n1 | tr [A-Z] [a-z] | sed -e 's/^.*: //;s/^sharp[- ]//'`
ORIENTATION=landscape
if [ "$MODEL" = "tosa" ] ||
   [ "$MODEL" = "poodle" ] || 
   [ "$MODEL" = "collie" ]; then
  ORIENTATION=portrait
else
  ORIENTATION=landscape
fi

mkdir -p /opt/QtPalmtop/lib/X11/kb # May not exist yet if the package is installed in a card, and not yet linked
mkdir -p /var/lib/xdm
mkdir -p /opt/QtPalmtop/lib/X11/xdm

ln -sf /opt/QtPalmtop/bin/Xqt-$ORIENTATION        /opt/QtPalmtop/bin/Xqt
ln -sf /opt/QtPalmtop/lib/X11/kb/xmodmaprc-$MODEL /opt/QtPalmtop/lib/X11/kb/xmodmaprc

ln -sf /opt/QtPalmtop/lib/X11/fonts /etc/fonts 
ln -sf /opt/QtPalmtop/lib/X11       /etc/X11

ln -sf /var/lib/xdm /opt/QtPalmtop/lib/X11/xdm/authdir

"""
      open(os.path.join(self.control_dir, "postinst"), "w").write(postinst)
      
      
      postrm   = """#!/bin/bash

rm /opt/QtPalmtop/bin/Xqt
rm -r /opt/QtPalmtop/lib/X11/kb

rm /etc/fonts
rm /etc/X11

rm -r /var/lib/xdm
rm -r /opt/QtPalmtop/lib/X11/xdm

"""
      open(os.path.join(self.control_dir, "postrm"), "w").write(postrm)
